/***********************************************************
 * $Id: fancysaxiostream.h 1334 2014-10-05 15:30:22Z fancylibs $
 *
 * FancySAX - XML serializer support
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSAXIOSTREAM_H_
#define FANCYSAXIOSTREAM_H_

#include <iostream>
#include <fancysax.h>
#include <fancyobject.h>

namespace Fancy {
namespace SAX {
namespace stream {

    FANCYSAX_API std::ostream &xmlQuote(std::ostream &o, const char *s);
    FANCYSAX_API std::ostream &xmlQuote(std::ostream &o, const std::string &s);
    FANCYSAX_API std::ostream &xmlAttrQuote(std::ostream &o, const std::string &s);

    class FANCYSAX_API XmlQuote
    {
      std::string str;
    public:
      XmlQuote(const char *s) : str(s) {}
      XmlQuote(const std::string &s) : str(s) {}

      const std::string &getStr() const { return str; }
    };

    // This allows us to write os << XmlQuote(str)
    inline std::ostream &operator<< (std::ostream &o, const XmlQuote &q)
    {
      return xmlQuote(o,q.getStr());
    }

    class FANCYSAX_API XmlAttrQuote
    {
      std::string str;
    public:
      XmlAttrQuote(const std::string &s) : str(s) {}

      const std::string &getStr() const { return str; }
    };

    // This allows us to write os << XmlQuote(str)
    inline std::ostream &operator<< (std::ostream &o, const XmlAttrQuote &q)
    {
      return xmlAttrQuote(o,q.getStr());
    }

    class FANCYSAX_API Attribute
    {
    public:
      Attribute(const std::string &k, const std::string &v)
        : key(k),value(v)
      {}

      Attribute(const char *k, const std::string &v)
        : key(k ? k : ""),value(v)
      {}

      Attribute(const std::string &k, const char *v)
        : key(k),value(v ? v : "")
      {}

      Attribute(const char *k, const char *v)
        : key(k ? k : ""),value(v ? v : "")
      {}

      Attribute(const char *k, const Fancy::ObjPtr<Fancy::Object> &obj);
      Attribute(const char *k, int v);
      Attribute(const char *k, unsigned v);
      Attribute(const char *k, int64_t v);
      Attribute(const char *k, uint64_t v);
      Attribute(const char *k, float v);
      Attribute(const char *k, double v);

      Attribute(const std::string &k, const Fancy::ObjPtr<Fancy::Object> &obj);
      Attribute(const std::string &k, int v);
      Attribute(const std::string &k, unsigned v);
      Attribute(const std::string &k, int64_t v);
      Attribute(const std::string &k, uint64_t v);
      Attribute(const std::string &k, float v);
      Attribute(const std::string &k, double v);

      const std::string &getKey() const { return key; }
      const std::string &getValue() const { return value; }
    private:
      std::string key;
      std::string value;
    };

    class FANCYSAX_API AttributeList
    {
    public:
      AttributeList();
      ~AttributeList();

      AttributeList &operator << (const Attribute &attr);

      std::string str() const;
    private:
      std::stringstream *s;
    };

    class FANCYSAX_API Tag
    {
    public:
      Tag(std::ostream &o, const std::string &t, bool empty = false);
      Tag(std::ostream &o, const std::string &t, const AttributeList &l, bool empty = false);
      Tag(std::ostream &o, const std::string &t, const Attribute &a, bool empty = false);

      Tag(const Tag &o, const std::string &t, bool empty = false);
      Tag(const Tag &o, const std::string &t, const AttributeList &l, bool empty = false);
      Tag(const Tag &o, const std::string &t, const Attribute &a, bool empty = false);

      ~Tag();

      // This is used in order to circumvent a parse error with gcc-3.3
      // in Expressions like: Tag(os,"abc") << "Hello, World!";
      Tag &me() { return *this; }
      std::ostream &getStream() const { return *os; }
      const std::string &getTag() const { return tag; }
    private:
      Tag(const Tag &);
      Tag &operator= (const Tag &);

      std::string tag;
      std::ostream *os;
    };

    inline Tag &operator<< (Tag &t, const Fancy::ObjPtr<Fancy::Object> &obj) {
        xmlQuote(t.getStream(),FANCY_OBJ_PTR_STRING(obj)); return t; }

    inline Tag &operator<< (Tag &t, const char *s)
    { xmlQuote(t.getStream(),s); return t; }

    inline Tag &operator<< (Tag &t, const std::string &s)
    { xmlQuote(t.getStream(),s); return t; }

    inline Tag &operator<< (Tag &t, char i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, short i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, unsigned short i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, int i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, unsigned i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, long i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, unsigned long i)
    { t.getStream() << i; return t; }

    inline Tag &operator<< (Tag &t, double i)
    { t.getStream() << i; return t; }
}}}

#endif /* FANCYSAXIOSTREAM_H_ */
